﻿using HarmonyLib;
using Steamworks;
using System.Collections.Generic;
using UnityEngine;

public class QuickUtilities : Mod
{
    private bool SailsOpen;
    private Semih_Network network;
    private bool MotorsOn;
    private bool AnchorsDown;
    static QuickUtilities instance;

    public void Start()
    {
        instance = this;

        Debug.Log("Quick-Utilities has been loaded!");
    }

    public void OnModUnload()
    {
        Debug.Log("Quick-Utilities has been unloaded!");
    }

    public void Anchor()
    {
        if (network == null)
            network = ComponentManager<Semih_Network>.Value;
        Anchor_Stationary[] anchors = FindObjectsOfType<Anchor_Stationary>();
        for (int i = 0; i < anchors.Length; i++)
        {
            if (anchors[i].AtBottom != AnchorsDown)
            {

                Message_NetworkBehaviour message = new Message_NetworkBehaviour(Messages.StationaryAnchorUse, anchors[i]);
                if (Semih_Network.IsHost)
                {
                    AccessTools.Method("Anchor_Stationary:Use", null, null).Invoke(anchors[i], null);
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
        }
        AnchorsDown = !AnchorsDown;
    }

    public void Sails()
    {
        if (network == null)
            network = ComponentManager<Semih_Network>.Value;
        List<Sail> allSails = Sail.AllSails;
        for (int i = 0; i < allSails.Count; i++)
        {
            // Close all sails 
            Debug.Log(SailsOpen);
            if (SailsOpen)
            {
                Message_NetworkBehaviour message = new Message_NetworkBehaviour(Messages.Sail_Close, allSails[i]);
                if (Semih_Network.IsHost)
                {
                    AccessTools.Method("Sail:Close", null, null).Invoke(allSails[i], null);
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                // Tell host to close mine
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
            // Open all sails
            else
            {
                Message_NetworkBehaviour message = new Message_NetworkBehaviour(Messages.Sail_Open, allSails[i]);
                if (Semih_Network.IsHost)
                {
                    allSails[i].Open();
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                // Tell the host to open 
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }

        }
        SailsOpen = !SailsOpen;
    }

    public void Engines()
    {
        if (network == null)
            network = ComponentManager<Semih_Network>.Value;
        // Finding all engines
        MotorWheel[] motors = FindObjectsOfType<MotorWheel>();
        for (int i = 0; i < motors.Length; i++)
        {
            // Checking if they are on already
            if (motors[i].MotorState != MotorsOn)
            {
                // If off then turn them on
                Message_NetworkBehaviour_ID message = new Message_NetworkBehaviour_ID(Messages.MotorWheel_PowerButton, network.NetworkIDManager, motors[i].ObjectIndex);
                if (Semih_Network.IsHost)
                {
                    motors[i].ToggleEngine();
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                    continue;
                }
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }

        }
        MotorsOn = !MotorsOn;
    }

    [ConsoleCommand(name: "anchor", docs: "")]
    public static void anchorCommand(string[] args)
    {
        instance.Anchor();
    }

    [ConsoleCommand(name: "sails", docs: "")]
    public static void sailsCommand(string[] args)
    {
        instance.Sails();
    }

    [ConsoleCommand(name: "engines", docs: "")]
    public static void enginesCommand(string[] args)
    {
        instance.Engines();
    }
}
